@@ -30,3 +30,20 @@ print.lmeresamp <- function(x, ci = FALSE, ...){
     confint(x)
   }
 }
+
+
+
+#' @title Print coefficients from a \code{coef_tbl} object
+#' 
+#' @description
+#' Print table of coefficients produced by \code{bootstrap_pvals}.
+#' 
+#' @param x the coef_tbl object to print
+print.coef_tbl <- function(x) {
+  cat(paste("Bootstrap type:", x$type, "\n"))
+  cat(paste("\n"))
+  cat(paste("Number of resamples:", x$B, "\n"))
+  cat(paste("\n"))
+  print(x$coefficients)
+  
+}
\ No newline at end of file
